@@ -12,7 +12,7 @@
     diffbc <- sum(abs(b-c))
     sumbc <- sum(b+c)
     bmin <- sum(pmin(b, c))
-    a <- sum(comm) - sum(colSums(comm) > 0)
+    a <- sum(comm > 0) - sum(colSums(comm) > 0)
     simpson <- bmin/(bmin + a)
     nest <- a/(bmin + a) * diffbc/(2*a + sumbc)
     sorensen <- sumbc/(2*a + sumbc)
@@ -28,7 +28,7 @@
     diffbc <- sum(abs(b-c))
     sumbc <- sum(b+c)
     bmin <- sum(pmin(b, c))
-    a <- sum(comm) - sum(colSums(comm) > 0)
+    a <- sum(comm > 0) - sum(colSums(comm) > 0)
     simpson <- 2*bmin/(2*bmin + a)
     nest <- a/(2*bmin + a) * diffbc/(a + sumbc)
     jaccard <- sumbc/(a + sumbc)
