@@ -1,7 +1,12 @@
 `rarecurve` <-
     function(x, step = 1, sample, xlab = "Sample Size", ylab = "Species",
-             label = TRUE, col, lty, ...)
+             label = TRUE, col, lty, tidy = FALSE, ...)
 {
+    ## matrix is faster than data.frame
+    x <- as.matrix(x)
+    ## check input data: must be counts
+    if (!identical(all.equal(x, round(x)), TRUE))
+        stop("function accepts only integers (counts)")
     ## sort out col and lty
     if (missing(col))
         col <- par("col")
@@ -9,6 +14,13 @@
         lty <- par("lty")
     tot <- rowSums(x)
     S <- specnumber(x)
+    ## remove empty rows or we fail
+    if (any(S <= 0)) {
+        message("empty rows removed")
+        x <- x[S > 0,, drop =FALSE]
+        tot <- tot[S > 0]
+        S <- S[S > 0]
+    }
     nr <- nrow(x)
     ## rep col and lty to appropriate length
     col <- rep(col, length.out = nr)
@@ -16,10 +28,23 @@
     ## Rarefy
     out <- lapply(seq_len(nr), function(i) {
         n <- seq(1, tot[i], by = step)
-        if (n[length(n)] != tot[i])
-            n <- c(n, tot[i])
+        if (n[length(n)] != tot[i]) {
+            ## don't want names on n an `c` adds a name from `tot[i]`)
+            n <- c(n, tot[i], use.names = FALSE)
+        }
         drop(rarefy(x[i,], n))
     })
+    ## instead of plotting a rarecurve, return a "tidy" data frame and
+    ## the let the user figure out how to display the results
+    if (tidy) {
+        len <- sapply(out, length)
+        nm <- rownames(x)
+        df <- data.frame(
+            "Site" = factor(rep(nm, len), levels=nm),
+            "Sample" = unlist(lapply(out, attr, which="Subsample")),
+            "Species" = unlist(out))
+        return(df) # exit with data.frame
+    }
     Nmax <- sapply(out, function(x) max(attr(x, "Subsample")))
     Smax <- sapply(out, max)
     ## set up plot
