@@ -1,5 +1,6 @@
-`scores.betadisper` <- function(x, display = c("sites", "centroids"),
-                                choices = c(1,2), ...)
+`scores.betadisper` <-
+    function(x, display = c("sites", "centroids"),
+             choices = c(1,2), ...)
 {
     display <- match.arg(display, several.ok = TRUE)
     sol <- list()
@@ -7,11 +8,11 @@
         sol$sites <- x$vectors[, choices]
     if("centroids" %in% display) {
         if(is.matrix(x$centroids))
-            sol$centroids <- x$centroids[, choices]
+            sol$centroids <- x$centroids[, choices, drop = FALSE]
         else
-            sol$centroids <- x$centroids[choices]
+            sol$centroids <- matrix(x$centroids[choices], ncol = length(choices), byrow = TRUE)
     }
     if (length(sol) == 1)
         sol <- sol[[1]]
-    return(sol)
+    sol
 }
