@@ -1,27 +1,25 @@
 `scores.cca` <-
-    function (x, choices = c(1, 2), display = c("sp", "wa", "cn"),
-              scaling = "species", hill = FALSE, ...)
+    function (x, choices = c(1, 2), display = c("sp", "wa", "bp", "cn"),
+              scaling = "species", hill = FALSE, tidy = FALSE, ...)
 {
-    if(inherits(x, "pcaiv")) {
-        warning("looks like ade4::cca object: you better use ade4 functions")
-        x <- ade2vegancca(x)
-    }
     ## Check the na.action, and pad the result with NA or WA if class
     ## "exclude"
     if (!is.null(x$na.action) && inherits(x$na.action, "exclude"))
         x <- ordiNApredict(x$na.action, x)
     tabula <- c("species", "sites", "constraints", "biplot",
-                "centroids")
-    names(tabula) <- c("sp", "wa", "lc", "bp", "cn")
+                "regression", "centroids")
+    names(tabula) <- c("sp", "wa", "lc", "bp", "reg", "cn")
     if (is.null(x$CCA))
         tabula <- tabula[1:2]
     display <- match.arg(display, c("sites", "species", "wa",
-                                    "lc", "bp", "cn"),
+                                    "lc", "bp", "reg", "cn", "all"),
                          several.ok = TRUE)
     if("sites" %in% display)
         display[display == "sites"] <- "wa"
     if("species" %in% display)
         display[display == "species"] <- "sp"
+    if("all" %in% display)
+        display <- names(tabula)
     take <- tabula[display]
     slam <- sqrt(c(x$CCA$eig, x$CA$eig)[choices])
     rnk <- x$CCA$rank
@@ -72,14 +70,25 @@
         rownames(b) <- rownames(x$CCA$biplot)
         if (scaling) {
             scal <- list(slam, 1, sqrt(slam))[[abs(scaling)]]
-            scal <- scal/max(scal) # scale proportionally to the "best" dim
             b <- sweep(b, 2, scal, "*")
         }
         sol$biplot <- b
     }
+    if ("regression" %in% take) {
+        b <- coef(x, norm = TRUE)
+        reg <- matrix(0, nrow(b), length(choices))
+        reg[, choices <= rnk] <- b[, choices[choices <= rnk]]
+        dimnames(reg) <- list(rownames(b),
+                              c(colnames(x$CCA$u), colnames(x$CA$u))[choices])
+        if (scaling) {
+            scal <- list(slam, 1, sqrt(slam))[[abs(scaling)]]
+            reg <- sweep(reg, 2, scal, "*")
+        }
+        sol$regression <- reg
+    }
     if ("centroids" %in% take) {
         if (is.null(x$CCA$centroids))
-            sol$centroids <- NA
+            sol$centroids <- NULL
         else {
             cn <- matrix(0, nrow(x$CCA$centroids), length(choices))
             cn[, choices <= rnk] <- x$CCA$centroids[, choices[choices <=
@@ -106,8 +115,37 @@
                              prefix = substr(names(sol)[i], 1, 3))
         }
     }
-    ## Only one type of scores: return a matrix instead of a list
-    if (length(sol) == 1)
-        sol <- sol[[1]]
-    sol
+    ## tidy scores
+    if (tidy) {
+        if (length(sol) == 0) # no requested scores existed
+            return(NULL)
+        ## re-group biplot arrays duplicating factor centroids
+        if (!is.null(sol$biplot) && !is.null(sol$centroids)) {
+            dup <- rownames(sol$biplot) %in% rownames(sol$centroids)
+            if (any(dup)) {
+                sol$factorbiplot <- sol$biplot[dup,, drop=FALSE]
+                sol$biplot <- sol$biplot[!dup,, drop=FALSE]
+            }
+        }
+        group <- sapply(sol, nrow)
+        group <- rep(names(group), group)
+        sol <- do.call(rbind, sol)
+        label <- rownames(sol)
+        rw <- x$rowsum # weights(x) can fail with na.action=na.exclude
+        cw <- weights(x, "species")
+        w <- rep(NA, nrow(sol))
+        if (any(weighted <- group == "sites"))
+            w[weighted] <- rw
+        if (any(weighted <- group == "constraints"))
+            w[weighted] <- rw
+        if (any(weighted <- group == "species"))
+            w[weighted] <- cw
+        sol <- as.data.frame(sol)
+        sol$score <- as.factor(group)
+        sol$label <- label
+        sol$weight <- w
+    }
+    ## return NULL instead of list(), and matrix instead of a list of
+    ## one matrix
+    switch(min(2, length(sol)), sol[[1]], sol)
 }
