@@ -2,25 +2,28 @@
 ### latter can have special features which are commented below. cca
 ### results are handled by scores.cca.
 `scores.rda` <-
-    function (x, choices = c(1, 2), display = c("sp", "wa", "cn"),
-              scaling = "species", const, correlation = FALSE, ...)
+    function (x, choices = c(1, 2), display = c("sp", "wa", "bp", "cn"),
+              scaling = "species", const, correlation = FALSE, tidy = FALSE,
+              ...)
 {
     ## Check the na.action, and pad the result with NA or WA if class
     ## "exclude"
     if (!is.null(x$na.action) && inherits(x$na.action, "exclude"))
         x <- ordiNApredict(x$na.action, x)
-    tabula <- c("species", "sites", "constraints", "biplot", 
-                "centroids")
-    names(tabula) <- c("sp", "wa", "lc", "bp", "cn")
-    if (is.null(x$CCA)) 
+    tabula <- c("species", "sites", "constraints", "biplot",
+                "regression", "centroids")
+    names(tabula) <- c("sp", "wa", "lc", "bp", "reg", "cn")
+    if (is.null(x$CCA))
         tabula <- tabula[1:2]
     display <- match.arg(display, c("sites", "species", "wa",
-                                    "lc", "bp", "cn"),
+                                    "lc", "bp", "cn", "reg"),
                          several.ok = TRUE)
     if("sites" %in% display)
       display[display == "sites"] <- "wa"
     if("species" %in% display)
       display[display == "species"] <- "sp"
+    if ("all" %in% display)
+        display <- names(tabula)
     take <- tabula[display]
     sumev <- x$tot.chi
     ## dbrda can have negative eigenvalues, but have scores only for
@@ -64,7 +67,10 @@
             }
             v <- const[1] * v
         }
-        sol$species <- v
+        if (nrow(v) > 0)
+            sol$species <- v
+        else
+            sol$species <- NULL
     }
     if ("sites" %in% take) {
         wa <- cbind(x$CCA$wa, x$CA$u)[, choices, drop=FALSE]
@@ -89,11 +95,27 @@
         b[, choices <= rnk] <- x$CCA$biplot[, choices[choices <= rnk]]
         colnames(b) <- c(colnames(x$CCA$u), colnames(x$CA$u))[choices]
         rownames(b) <- rownames(x$CCA$biplot)
+        if (scaling) {
+            scal <- list(slam, 1, sqrt(slam))[[abs(scaling)]]
+            b <- sweep(b, 2, scal, "*")
+        }
         sol$biplot <- b
     }
+    if ("regression" %in% take) {
+        b <- coef(x, norm = TRUE)
+        reg <- matrix(0, nrow(b), length(choices))
+        reg[, choices <= rnk] <- b[, choices[choices <= rnk]]
+        dimnames(reg) <- list(rownames(b),
+                              c(colnames(x$CCA$u), colnames(x$CA$u))[choices])
+        if (scaling) {
+            scal <- list(slam, 1, sqrt(slam))[[abs(scaling)]]
+            reg <- sweep(reg, 2, scal, "*")
+        }
+        sol$regression <- reg
+    }
     if ("centroids" %in% take) {
         if (is.null(x$CCA$centroids))
-            sol$centroids <- NA
+            sol$centroids <- NULL
         else {
             cn <- matrix(0, nrow(x$CCA$centroids), length(choices))
             cn[, choices <= rnk] <- x$CCA$centroids[, choices[choices <= rnk]]
@@ -110,18 +132,39 @@
     ## Take care that scores have names
     if (length(sol)) {
         for (i in seq_along(sol)) {
-            if (is.matrix(sol[[i]])) 
+            if (is.matrix(sol[[i]]))
                 rownames(sol[[i]]) <-
-                    rownames(sol[[i]], do.NULL = FALSE, 
+                    rownames(sol[[i]], do.NULL = FALSE,
                              prefix = substr(names(sol)[i], 1, 3))
         }
     }
-    ## Only one type of scores: return a matrix instead of a list
-    if (length(sol) == 1) 
-        sol <- sol[[1]]
+    ## tidy scores
+    if (tidy) {
+        if (length(sol) == 0) # no scores
+            return(NULL)
+        ## re-group biplot arrays duplicating factor centroids
+        if (!is.null(sol$biplot) && !is.null(sol$centroids)) {
+            dup <- rownames(sol$biplot) %in% rownames(sol$centroids)
+            if (any(dup)) {
+                sol$factorbiplot <- sol$biplot[dup,, drop=FALSE]
+                sol$biplot <- sol$biplot[!dup,, drop=FALSE]
+            }
+        }
+        group <- sapply(sol, nrow)
+        group <- rep(names(group), group)
+        sol <- do.call(rbind, sol)
+        label <- rownames(sol)
+        sol <- as.data.frame(sol)
+        sol$score <- as.factor(group)
+        sol$label <- label
+    }
     ## collapse const if both items identical
     if (identical(const[1], const[2]))
         const <- const[1]
-    attr(sol, "const") <- const
+    ## return NULL for list(), matrix for single scores, and a list
+    ## for several scores
+    sol <- switch(min(2, length(sol)), sol[[1]], sol)
+    if (!is.null(sol))
+        attr(sol, "const") <- const
     sol
 }
