@@ -5,7 +5,7 @@
     METHODS <- c("collector", "random", "exact", "rarefaction", "coleman")
     method <- match.arg(method, METHODS)
     if (!is.null(w) && !(method %in% c("random", "collector")))
-        stop(gettextf("weights 'w' can be only used with methods 'random' and 'collector'"))
+        stop("weights 'w' can be only used with methods 'random' and 'collector'")
     if (!missing(subset)) {
         comm <- subset(comm, subset)
         w <- subset(w, subset)
@@ -15,34 +15,31 @@
     x <- x[, colSums(x) > 0, drop=FALSE]
     n <- nrow(x)
     p <- ncol(x)
-    if (p == 1) {
-        x <- t(x)
-        n <- nrow(x)
-        p <- ncol(x)
-    }
     accumulator <- function(x, ind) {
-        rowSums(apply(x[ind, ], 2, cumsum) > 0)
+        rowSums(apply(x[ind, , drop=FALSE], 2, cumsum) > 0)
     }
     specaccum <- sdaccum <- sites <- perm <- NULL
     if (n == 1 && method != "rarefaction")
-        message("No actual accumulation since only 1 site provided")
+        message("no actual accumulation since only one site provided")
     switch(method, collector = {
-        sites <- 1:n
+        sites <- seq_len(n)
         xout <- weights <- cumsum(w)
         specaccum <- accumulator(x, sites)
+        perm <- as.matrix(specaccum)
+        weights <- as.matrix(weights)
     }, random = {
         permat <- getPermuteMatrix(permutations, n)
         perm <- apply(permat, 1, accumulator, x = x)
         if (!is.null(w))
-            weights <- apply(permat, 1, function(i) cumsum(w[i]))
-        sites <- 1:n
+            weights <- as.matrix(apply(permat, 1, function(i) cumsum(w[i])))
+        sites <- seq_len(n)
         if (is.null(w)) {
             specaccum <- apply(perm, 1, mean)
             sdaccum <- apply(perm, 1, sd)
         } else {
             sumw <- sum(w)
             xout <- seq(sumw/n, sumw, length.out = n)
-            intx <- sapply(seq_len(n), function(i)
+            intx <- sapply(seq_len(NCOL(perm)), function(i)
                            approx(weights[,i], perm[,i], xout = xout)$y)
             specaccum <- apply(intx, 1, mean)
             sdaccum <- apply(intx, 1, sd)
@@ -97,9 +94,9 @@
             result[i, ] <- (1 - i/n)^freq
         }
         result <- 1 - result
-        sites <- 1:n
-        specaccum <- apply(result, 1, sum)
-        sdaccum <- sqrt(apply(result * (1 - result), 1, sum))
+        sites <- seq_len(n)
+        specaccum <- rowSums(result)
+        sdaccum <- sqrt(rowSums(result * (1 - result)))
     })
     out <- list(call = match.call(), method = method, sites = sites,
                 richness = specaccum, sd = sdaccum, perm = perm)
@@ -109,6 +106,9 @@
     }
     if (method == "rarefaction")
         out$individuals <- ind
+    ## return 'freq' for methods that are solely defined by them
+    if (method %in% c("exact", "rarefaction", "coleman"))
+        out$freq <- freq
     if (method == "random")
         attr(out, "control") <- attr(permat, "control")
     class(out) <- "specaccum"
